@@ -36,7 +36,8 @@
 using girerr::error;
 #include "xmlrpc-c/base.h"
 #include "xmlrpc-c/base_int.h"
-#include "env_wrap.hpp"
+#include "xmlrpc-c/string_int.h"
+#include "xmlrpc-c/env_wrap.hpp"
 
 #include "xmlrpc-c/base.hpp"
 
@@ -58,6 +59,12 @@ class cDatetimeValueWrapper {
 public:
     xmlrpc_value * valueP;
     
+    cDatetimeValueWrapper(xmlrpc_datetime const cppvalue) {
+        env_wrap env;
+        
+        this->valueP = xmlrpc_datetime_new(&env.env_c, cppvalue);
+        throwIfError(env);
+    }
     cDatetimeValueWrapper(time_t const cppvalue) {
         env_wrap env;
         
@@ -249,6 +256,15 @@ value::type() const {
 
 
 
+ostream& operator<<(ostream& out, value::type_t const& type) {
+
+    string typeName;
+
+    return out << string(xmlrpc_type_name((xmlrpc_type)type));
+}
+
+
+
 value_int::value_int(int const cppvalue) {
 
     class cWrapper {
@@ -448,6 +464,15 @@ value_datetime::value_datetime(string const cppvalue) {
 
 
 
+value_datetime::value_datetime(xmlrpc_datetime const cppvalue) {
+
+    cDatetimeValueWrapper wrapper(cppvalue);
+    
+    this->instantiate(wrapper.valueP);
+}
+
+
+
 value_datetime::value_datetime(time_t const cppvalue) {
 
     cDatetimeValueWrapper wrapper(cppvalue);
@@ -490,6 +515,21 @@ value_datetime::value_datetime(xmlrpc_c::value const baseValue) {
 
 
 
+value_datetime::operator xmlrpc_datetime() const {
+
+    this->validateInstantiated();
+
+    xmlrpc_datetime retval;
+    env_wrap env;
+
+    xmlrpc_read_datetime(&env.env_c, this->cValueP, &retval);
+    throwIfError(env);
+
+    return retval;
+}
+
+
+
 value_datetime::operator time_t() const {
 
     this->validateInstantiated();
@@ -549,6 +589,28 @@ value_datetime::cvalue() const {
 
 
 
+string
+value_datetime::iso8601Value() const {
+
+    string retval;
+
+    this->validateInstantiated();
+
+    const char * iso8601;
+    env_wrap env;
+
+    xmlrpc_read_datetime_8601(&env.env_c, this->cValueP, &iso8601);
+    throwIfError(env);
+
+    retval = iso8601;
+
+    xmlrpc_strfree(iso8601);
+
+    return retval;
+}
+
+
+
 class cNewStringWrapper {
 public:
     xmlrpc_value * valueP;
